@@ -48,6 +48,16 @@ class IncompatibleTagsException(Exception):
             f" {[tag.value for tag in second_set.filter(TagType.REJECT)]}.")
 
 
+class TryNextDestinationOrFail(Exception):
+    # Try next destination, fail job if destination options exhausted
+    pass
+
+
+class TryNextDestinationOrWait(Exception):
+    # Try next destination, raise JobNotReadyException if destination options exhausted
+    pass
+
+
 class TagSetManager(object):
 
     def __init__(self, tags=[]):
@@ -410,6 +420,7 @@ class EntityWithRules(Entity):
 
     def evaluate_early(self, context):
         new_entity = copy.deepcopy(self)
+        context.update(new_entity.context or {})
         for rule in self.rules.values():
             if rule.is_matching(context):
                 rule = rule.evaluate_early(context)
